@@ -1,19 +1,21 @@
 from tqdm import tqdm
 import numpy as np
 import torch
-from typing import Callable, Optional, Union
-from alibi_detect.cd.base_online import BaseDriftOnline
+from typing import Any, Callable, Optional, Union
+from alibi_detect.cd.base_online import BaseMultiDriftOnline
+from alibi_detect.utils.pytorch import get_device
 from alibi_detect.utils.pytorch.kernels import GaussianRBF
 from alibi_detect.utils.pytorch import zero_diag, quantile
 
 
-class MMDDriftOnlineTorch(BaseDriftOnline):
+class MMDDriftOnlineTorch(BaseMultiDriftOnline):
     def __init__(
             self,
             x_ref: Union[np.ndarray, list],
             ert: float,
             window_size: int,
             preprocess_fn: Optional[Callable] = None,
+            x_ref_preprocessed: bool = False,
             kernel: Callable = GaussianRBF,
             sigma: Optional[np.ndarray] = None,
             n_bootstraps: int = 1000,
@@ -30,13 +32,18 @@ class MMDDriftOnlineTorch(BaseDriftOnline):
         x_ref
             Data used as reference distribution.
         ert
-            The expected run-time (ERT) in the absence of drift.
+            The expected run-time (ERT) in the absence of drift. For the multivariate detectors, the ERT is defined
+            as the expected run-time from t=0.
         window_size
             The size of the sliding test-window used to compute the test-statistic.
             Smaller windows focus on responding quickly to severe drift, larger windows focus on
             ability to detect slight drift.
         preprocess_fn
             Function to preprocess the data before computing the data drift metrics.
+        x_ref_preprocessed
+            Whether the given reference data `x_ref` has been preprocessed yet. If `x_ref_preprocessed=True`, only
+            the test data `x` will be preprocessed at prediction time. If `x_ref_preprocessed=False`, the reference
+            data will also be preprocessed.
         kernel
             Kernel used for the MMD computation, defaults to Gaussian RBF kernel.
         sigma
@@ -62,6 +69,7 @@ class MMDDriftOnlineTorch(BaseDriftOnline):
             ert=ert,
             window_size=window_size,
             preprocess_fn=preprocess_fn,
+            x_ref_preprocessed=x_ref_preprocessed,
             n_bootstraps=n_bootstraps,
             verbose=verbose,
             input_shape=input_shape,
@@ -69,16 +77,12 @@ class MMDDriftOnlineTorch(BaseDriftOnline):
         )
         self.meta.update({'backend': 'pytorch'})
 
-        # set backend
-        if device is None or device.lower() in ['gpu', 'cuda']:
-            self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
-            if self.device.type == 'cpu':
-                print('No GPU detected, fall back on CPU.')
-        else:
-            self.device = torch.device('cpu')
+        # set device
+        self.device = get_device(device)
 
         # initialize kernel
-        sigma = torch.from_numpy(sigma).to(self.device) if isinstance(sigma, np.ndarray) else None
+        sigma = torch.from_numpy(sigma).to(self.device) if isinstance(sigma,  # type: ignore[assignment]
+                                                                      np.ndarray) else None
         self.kernel = kernel(sigma) if kernel == GaussianRBF else kernel
 
         # compute kernel matrix for the reference data
@@ -89,7 +93,7 @@ class MMDDriftOnlineTorch(BaseDriftOnline):
         self._initialise()
 
     def _configure_ref_subset(self):
-        etw_size = 2*self.window_size-1  # etw = extended test window
+        etw_size = 2 * self.window_size - 1  # etw = extended test window
         rw_size = self.n - etw_size  # rw = ref-window
         # Make split and ensure it doesn't cause an initial detection
         mmd_init = None
@@ -100,13 +104,13 @@ class MMDDriftOnlineTorch(BaseDriftOnline):
             self.test_window = self.x_ref[self.init_test_inds]
             # Compute initial mmd to check for initial detection
             self.k_xx_sub = self.k_xx[self.ref_inds][:, self.ref_inds]
-            self.k_xx_sub_sum = zero_diag(self.k_xx_sub).sum()/(rw_size*(rw_size-1))
+            self.k_xx_sub_sum = zero_diag(self.k_xx_sub).sum() / (rw_size * (rw_size - 1))
             self.k_xy = self.kernel(self.x_ref[self.ref_inds], self.test_window)
             k_yy = self.kernel(self.test_window, self.test_window)
             mmd_init = (
-                self.k_xx_sub_sum +
-                zero_diag(k_yy).sum()/(self.window_size*(self.window_size-1)) -
-                2*self.k_xy.mean()
+                    self.k_xx_sub_sum +
+                    zero_diag(k_yy).sum() / (self.window_size * (self.window_size - 1)) -
+                    2 * self.k_xy.mean()
             )
 
     def _configure_thresholds(self):
@@ -116,7 +120,7 @@ class MMDDriftOnlineTorch(BaseDriftOnline):
         # test windows of size W (so 2W-1 test samples in total)
 
         w_size = self.window_size
-        etw_size = 2*w_size-1  # etw = extended test window
+        etw_size = 2 * w_size - 1  # etw = extended test window
         rw_size = self.n - etw_size  # rw = sub-ref window
 
         perms = [torch.randperm(self.n) for _ in range(self.n_bootstraps)]
@@ -135,28 +139,28 @@ class MMDDriftOnlineTorch(BaseDriftOnline):
         k_xy_col_sums_all = [
             self.k_xx[x_inds][:, y_inds].sum(0) for x_inds, y_inds in
             (tqdm(zip(x_inds_all, y_inds_all), total=self.n_bootstraps) if self.verbose else
-                zip(x_inds_all, y_inds_all))
+             zip(x_inds_all, y_inds_all))
         ]
         k_xx_sums_all = [(
-            k_full_sum - zero_diag(self.k_xx[y_inds][:, y_inds]).sum() - 2*k_xy_col_sums.sum()
-        )/(rw_size*(rw_size-1)) for y_inds, k_xy_col_sums in zip(y_inds_all, k_xy_col_sums_all)]
-        k_xy_col_sums_all = [k_xy_col_sums/(rw_size*w_size) for k_xy_col_sums in k_xy_col_sums_all]
+                                 k_full_sum - zero_diag(self.k_xx[y_inds][:, y_inds]).sum() - 2 * k_xy_col_sums.sum()
+                         ) / (rw_size * (rw_size - 1)) for y_inds, k_xy_col_sums in zip(y_inds_all, k_xy_col_sums_all)]
+        k_xy_col_sums_all = [k_xy_col_sums / (rw_size * w_size) for k_xy_col_sums in k_xy_col_sums_all]
 
         # Now to iterate through the W overlapping windows
         thresholds = []
         p_bar = tqdm(range(w_size), "Computing thresholds") if self.verbose else range(w_size)
         for w in p_bar:
-            y_inds_all_w = [y_inds[w:w+w_size] for y_inds in y_inds_all]  # test windows of size w_size
+            y_inds_all_w = [y_inds[w:w + w_size] for y_inds in y_inds_all]  # test windows of size w_size
             mmds = [(
-                k_xx_sum +
-                zero_diag(self.k_xx[y_inds_w][:, y_inds_w]).sum()/(w_size*(w_size-1)) -
-                2*k_xy_col_sums[w:w+w_size].sum()
-            ) for k_xx_sum, y_inds_w, k_xy_col_sums in zip(k_xx_sums_all, y_inds_all_w, k_xy_col_sums_all)
-            ]
+                    k_xx_sum +
+                    zero_diag(self.k_xx[y_inds_w][:, y_inds_w]).sum() / (w_size * (w_size - 1)) -
+                    2 * k_xy_col_sums[w:w + w_size].sum())
+                    for k_xx_sum, y_inds_w, k_xy_col_sums in zip(k_xx_sums_all, y_inds_all_w, k_xy_col_sums_all)
+                    ]
             mmds = torch.tensor(mmds)  # an mmd for each bootstrap sample
 
             # Now we discard all bootstrap samples for which mmd is in top (1/ert)% and record the thresholds
-            thresholds.append(quantile(mmds, 1-self.fpr))
+            thresholds.append(quantile(mmds, 1 - self.fpr))
             y_inds_all = [y_inds_all[i] for i in range(len(y_inds_all)) if mmds[i] < thresholds[-1]]
             k_xx_sums_all = [
                 k_xx_sums_all[i] for i in range(len(k_xx_sums_all)) if mmds[i] < thresholds[-1]
@@ -167,30 +171,32 @@ class MMDDriftOnlineTorch(BaseDriftOnline):
 
         self.thresholds = thresholds
 
-    def score(self, x_t: np.ndarray) -> Union[float, None]:
+    def _update_state(self, x_t: torch.Tensor):  # type: ignore[override]
+        self.t += 1
+        kernel_col = self.kernel(self.x_ref[self.ref_inds], x_t)
+        self.test_window = torch.cat([self.test_window[(1 - self.window_size):], x_t], 0)
+        self.k_xy = torch.cat([self.k_xy[:, (1 - self.window_size):], kernel_col], 1)
+
+    def score(self, x_t: Union[np.ndarray, Any]) -> float:
         """
         Compute the test-statistic (squared MMD) between the reference window and test window.
-        If the test-window is not yet full then a test-statistic of None is returned.
 
         Parameters
         ----------
         x_t
-            A single instance.
+            A single instance to be added to the test-window.
 
         Returns
         -------
-        Squared MMD estimate between reference window and test window
+        Squared MMD estimate between reference window and test window.
         """
-        x_t = torch.from_numpy(x_t[None, :]).to(self.device)
-        kernel_col = self.kernel(self.x_ref[self.ref_inds], x_t)
-
-        self.test_window = torch.cat([self.test_window[(1-self.window_size):], x_t], 0)
-        self.k_xy = torch.cat([self.k_xy[:, (1-self.window_size):], kernel_col], 1)
+        x_t = super()._preprocess_xt(x_t)
+        x_t = torch.from_numpy(x_t).to(self.device)
+        self._update_state(x_t)
         k_yy = self.kernel(self.test_window, self.test_window)
         mmd = (
-            self.k_xx_sub_sum +
-            zero_diag(k_yy).sum()/(self.window_size*(self.window_size-1)) -
-            2*self.k_xy.mean()
+                self.k_xx_sub_sum +
+                zero_diag(k_yy).sum() / (self.window_size * (self.window_size - 1)) -
+                2 * self.k_xy.mean()
         )
-
         return float(mmd.detach().cpu())
