@@ -1,10 +1,4 @@
-try:
-    from fbprophet import Prophet
-
-    PROPHET_INSTALLED = True
-except ImportError:
-    PROPHET_INSTALLED = False
-
+from fbprophet import Prophet
 import logging
 import pandas as pd
 from typing import Dict, List, Union
@@ -117,8 +111,9 @@ class OutlierProphet(BaseDetector, FitMixin):
         self.cap = cap
 
         # set metadata
-        self.meta['detector_type'] = 'offline'
+        self.meta['detector_type'] = 'outlier'
         self.meta['data_type'] = 'time-series'
+        self.meta['online'] = False
 
     def fit(self, df: pd.DataFrame) -> None:
         """
