@@ -69,8 +69,9 @@ class OutlierVAE(BaseDetector, FitMixin, ThresholdMixin):
                             'or `encoder_net` and `decoder_net` (tf.keras.Sequential).')
 
         # set metadata
-        self.meta['detector_type'] = 'offline'
+        self.meta['detector_type'] = 'outlier'
         self.meta['data_type'] = data_type
+        self.meta['online'] = False
 
     def fit(self,
             X: np.ndarray,
